var __classPrivateFieldGet = (this && this.__classPrivateFieldGet) || function (receiver, state, kind, f) {
    if (kind === "a" && !f) throw new TypeError("Private accessor was defined without a getter");
    if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError("Cannot read private member from an object whose class did not declare it");
    return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
};
var _Server_ended;
import { logger } from "@libp2p/logger";
import { pushable } from "it-pushable";
import * as lp from "it-length-prefixed";
import { pipe } from "it-pipe";
import { assert, SECONDS } from "@canvas-js/utils";
import * as Sync from "@canvas-js/gossiplog/protocols/sync";
import { decodeKey, encodeNode } from "./utils.js";
export async function* encodeResponses(responses) {
    for await (const res of responses) {
        yield Sync.Response.encode(res);
    }
}
export async function* decodeRequests(requests) {
    for await (const msg of requests) {
        const res = Sync.Request.decode(msg.subarray());
        yield res;
    }
}
export class Server {
    static async handleStream(txn, stream) {
        const server = new Server(txn, stream);
        const signal = AbortSignal.timeout(Server.timeout);
        const onAbort = () => server.abort();
        signal.addEventListener("abort", onAbort);
        try {
            await pipe(stream, lp.decode, decodeRequests, server, encodeResponses, lp.encode, stream);
        }
        catch (err) {
            server.log.error(err);
        }
        finally {
            signal.removeEventListener("abort", onAbort);
        }
    }
    constructor(txn, stream) {
        Object.defineProperty(this, "txn", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: txn
        });
        Object.defineProperty(this, "source", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: pushable({ objectMode: true })
        });
        Object.defineProperty(this, "log", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: void 0
        });
        _Server_ended.set(this, false);
        Object.defineProperty(this, "sink", {
            enumerable: true,
            configurable: true,
            writable: true,
            value: async (reqs) => {
                for await (const req of reqs) {
                    if (__classPrivateFieldGet(this, _Server_ended, "f")) {
                        return;
                    }
                    const res = await this.handleRequest(req);
                    if (__classPrivateFieldGet(this, _Server_ended, "f")) {
                        return;
                    }
                    this.source.push(res);
                }
                this.source.end();
            }
        });
        this.log = logger(`canvas:sync:server:${stream.id}`);
    }
    abort() {
        this.source.push({ abort: { cooldown: 0 } });
        this.source.end();
    }
    async handleRequest(req) {
        this.log.trace("handling", req);
        if (req.getRoot !== undefined) {
            const root = await this.txn.getRoot();
            return { getRoot: { root: encodeNode(root) } };
        }
        else if (req.getNode !== undefined) {
            const { level, key } = req.getNode;
            assert(level !== null && level !== undefined, "missing level in getNode request");
            const node = await this.txn.getNode(level, decodeKey(key));
            if (node === null) {
                return { getNode: {} };
            }
            else {
                return { getNode: { node: encodeNode(node) } };
            }
        }
        else if (req.getChildren !== undefined) {
            const { level, key } = req.getChildren;
            assert(level !== null && level !== undefined, "missing level in getChildren request");
            const children = await this.txn.getChildren(level, decodeKey(key));
            return { getChildren: { children: children.map(encodeNode) } };
        }
        else if (req.getValues !== undefined) {
            const { keys } = req.getValues;
            const values = await this.txn.getValues(keys);
            return { getValues: { values } };
        }
        else {
            throw new Error("invalid request type");
        }
    }
}
_Server_ended = new WeakMap();
Object.defineProperty(Server, "timeout", {
    enumerable: true,
    configurable: true,
    writable: true,
    value: 2 * SECONDS
});
